@@ -45,6 +45,9 @@
 #include "error.h"
 #include "rwfile.h"
 
+// Whether to print detailed information about TigEntry, TigContent instances throughout their lifecycle.
+//#define TRACE_CONTENT_INSTANCES
+
 #define WRITEBUFFERSIZE (8192)
 
 /**
@@ -69,10 +72,14 @@ TIEXPORT2 TigEntry* TICALL tifiles_te_create(const char *filename, FileClass typ
 			entry->filename = g_path_get_basename(filename);
 			entry->type = type;
 
-			if(type == TIFILE_FLASH)
+			if (type == TIFILE_FLASH)
+			{
 				entry->content.flash = tifiles_content_create_flash(model);
-			else if(type & TIFILE_REGULAR)
+			}
+			else if (type & TIFILE_REGULAR)
+			{
 				entry->content.regular = tifiles_content_create_regular(model);
+			}
 		}
 	}
 	else
@@ -80,6 +87,10 @@ TIEXPORT2 TigEntry* TICALL tifiles_te_create(const char *filename, FileClass typ
 		tifiles_critical("%s: invalid filename", __FUNCTION__);
 	}
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_te_create: %p", entry);
+	tifiles_te_display(entry);
+#endif
 
 	return entry;
 }
@@ -90,18 +101,27 @@ TIEXPORT2 TigEntry* TICALL tifiles_te_create(const char *filename, FileClass typ
  *
  * Destroy a #TigEntry structure as well as fields.
  *
- * Return value: none.
+ * Return value: always 0.
  **/
 TIEXPORT2 int TICALL tifiles_te_delete(TigEntry* entry)
 {
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_te_delete: %p", entry);
+	tifiles_te_display(entry);
+#endif
+
 	if (entry != NULL)
 	{
 		g_free(entry->filename);
 
-		if(entry->type == TIFILE_FLASH)
+		if (entry->type == TIFILE_FLASH)
+		{
 			tifiles_content_delete_flash(entry->content.flash);
-		else if(entry->type & TIFILE_REGULAR)
+		}
+		else if (entry->type & TIFILE_REGULAR)
+		{
 			tifiles_content_delete_regular(entry->content.regular);
+		}
 
 		g_free(entry);
 	}
@@ -113,16 +133,51 @@ TIEXPORT2 int TICALL tifiles_te_delete(TigEntry* entry)
 	return 0;
 }
 
+/**
+ * tifiles_te_display:
+ * @entry: a #TigEntry structure pointer.
+ *
+ * Display a #TigEntry structure's contents.
+ *
+ * Return value: an error code, 0 otherwise.
+ **/
+TIEXPORT2 int TICALL tifiles_te_display(TigEntry* entry)
+{
+	if (entry == NULL)
+	{
+		tifiles_critical("%s(NULL)", __FUNCTION__);
+		return ERR_INVALID_FILE;
+	}
+
+	tifiles_info("Filename:          %s", entry->filename);
+	tifiles_info("File class:        %04X (%u)", entry->type, entry->type);
+
+	if (entry->type == TIFILE_FLASH)
+	{
+		tifiles_file_display_flash(entry->content.flash);
+	}
+	else if (entry->type & TIFILE_REGULAR)
+	{
+		tifiles_file_display_regular(entry->content.regular);
+	}
+	else
+	{
+		tifiles_info("Data:              %p", entry->content.data);
+	}
+
+	return 0;
+}
+
 /**
  * tifiles_te_create_array:
  * @nelts: size of NULL-terminated array (number of #TigEntry structures).
  *
  * Allocate a NULL-terminated array of #TigEntry structures. You have to allocate
- * each elements of the array by yourself.
+ * each element of the array by yourself.
  *
  * Return value: the array or NULL if error.
  **/
-TIEXPORT2 TigEntry**	TICALL tifiles_te_create_array(int nelts)
+TIEXPORT2 TigEntry**	TICALL tifiles_te_create_array(unsigned int nelts)
 {
 	return g_malloc0((nelts + 1) * sizeof(TigEntry *));
 }
@@ -133,13 +188,18 @@ TIEXPORT2 TigEntry**	TICALL tifiles_te_create_array(int nelts)
  * @nelts: size of NULL-terminated array (number of #TigEntry structures).
  *
  * Re-allocate a NULL-terminated array of #TigEntry structures. You have to allocate
- * each elements of the array by yourself.
+ * each element of the array by yourself.
  *
  * Return value: the array or NULL if error.
  **/
-TIEXPORT2 TigEntry**	TICALL tifiles_te_resize_array(TigEntry** array, int nelts)
+TIEXPORT2 TigEntry**	TICALL tifiles_te_resize_array(TigEntry** array, unsigned int nelts)
 {
-	return g_realloc(array, (nelts + 1) * sizeof(TigEntry *));
+	TigEntry ** ptr = g_realloc(array, (nelts + 1) * sizeof(TigEntry *));
+	if (ptr != NULL)
+	{
+		ptr[nelts] = NULL;
+	}
+	return ptr;
 }
 
 /**
@@ -154,10 +214,16 @@ TIEXPORT2 void			TICALL tifiles_te_delete_array(TigEntry** array)
 {
 	TigEntry** ptr;
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_te_delete_array: %p", array);
+#endif
+
 	if (array != NULL)
 	{
-		for(ptr = array; ptr; ptr++)
+		for (ptr = array; *ptr; ptr++)
+		{
 			tifiles_te_delete(*ptr);
+		}
 		g_free(array);
 	}
 	else
@@ -183,7 +249,7 @@ TIEXPORT2 int TICALL tifiles_te_sizeof_array(TigEntry** array)
 
 	if (array != NULL)
 	{
-		for(p = array; *p; p++, i++);
+		for (p = array; *p; p++, i++);
 	}
 	else
 	{
@@ -213,7 +279,7 @@ TIEXPORT2 int TICALL tifiles_content_add_te(TigContent *content, TigEntry *te)
 		return 0;
 	}
 
-	if(te->type == TIFILE_FLASH)
+	if (te->type == TIFILE_FLASH)
 	{
 		int n = content->n_apps;
 
@@ -225,7 +291,7 @@ TIEXPORT2 int TICALL tifiles_content_add_te(TigContent *content, TigEntry *te)
 
 		return n;
 	}
-	else if(te->type & TIFILE_REGULAR)
+	else if (te->type & TIFILE_REGULAR)
 	{
 		int n = content->n_vars;
 
@@ -252,7 +318,7 @@ TIEXPORT2 int TICALL tifiles_content_add_te(TigContent *content, TigEntry *te)
  **/
 TIEXPORT2 int TICALL tifiles_content_del_te(TigContent *content, TigEntry *te)
 {
-	int i, j, k;
+	unsigned int i, j, k;
 
 	if (content == NULL || te == NULL)
 	{
@@ -260,36 +326,50 @@ TIEXPORT2 int TICALL tifiles_content_del_te(TigContent *content, TigEntry *te)
 		return -1;
 	}
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_content_del_te: %p %p", content, te);
+	tifiles_file_display_tigcontent(content);
+	tifiles_te_display(te);
+#endif
+
 	// Search for entry
-	for(i = 0; i < content->n_vars && (te->type & TIFILE_REGULAR); i++)
+	for (i = 0; i < content->n_vars && (te->type & TIFILE_REGULAR); i++)
 	{
 		TigEntry *s = content->var_entries[i];
 
-		if(!strcmp(s->filename, te->filename))
+		if (!strcmp(s->filename, te->filename))
+		{
 			break;
+		}
 	}
 
-	for(j = 0; j < content->n_apps && (te->type & TIFILE_FLASH); j++)
+	for (j = 0; j < content->n_apps && (te->type & TIFILE_FLASH); j++)
 	{
 		TigEntry *s = content->app_entries[i];
 
-		if(!strcmp(s->filename, te->filename))
+		if (!strcmp(s->filename, te->filename))
+		{
 			break;
+		}
 	}
 
 	// Not found ? Exit !
-	if((i == content->n_vars) && (j == content->n_apps))
+	if ((i == content->n_vars) && (j == content->n_apps))
+	{
 		return -1;
+	}
 
 	// Release
-	if(i < content->n_vars)
+	if (i < content->n_vars)
 	{
 		// Delete
 		tifiles_te_delete(content->var_entries[i]);
 
 		// And shift
-		for(k = i; k < content->n_vars; k++)
+		for (k = i; k < content->n_vars; k++)
+		{
 			content->var_entries[k] = content->var_entries[k+1];
+		}
 		content->var_entries[k] = NULL;
 
 		// And resize
@@ -299,14 +379,16 @@ TIEXPORT2 int TICALL tifiles_content_del_te(TigContent *content, TigEntry *te)
 		return content->n_vars;
 	}
 
-	if(j < content->n_apps)
+	if (j < content->n_apps)
 	{
 		// Delete
 		tifiles_te_delete(content->app_entries[j]);
 
 		// And shift
-		for(k = j; k < content->n_apps; k++)
+		for (k = j; k < content->n_apps; k++)
+		{
 			content->app_entries[k] = content->app_entries[k+1];
+		}
 		content->app_entries[k] = NULL;
 
 		// And resize
@@ -347,52 +429,72 @@ TIEXPORT2 int TICALL tifiles_tigroup_add_file(const char *src_filename, const ch
 	}
 
 	// group file is created if non existent
-	if(!stricmp(tifiles_fext_get(dst_filename), "tig"))
+	if (!stricmp(tifiles_fext_get(dst_filename), "tig"))
 	{
-		if(!g_file_test(dst_filename, G_FILE_TEST_EXISTS))
+		if (!g_file_test(dst_filename, G_FILE_TEST_EXISTS))
 		{
 			content = tifiles_content_create_tigroup(CALC_NONE, 0);
 			tifiles_file_write_tigroup(dst_filename, content);
 			tifiles_content_delete_tigroup(content);
+			content = NULL;
 		}
 	}
 
 	// src can't be a TiGroup file but dst should be
-	if(!(tifiles_file_is_ti(src_filename) && !tifiles_file_is_tigroup(src_filename) &&
+	if (!(tifiles_file_is_ti(src_filename) && !tifiles_file_is_tigroup(src_filename) &&
 		tifiles_file_is_tigroup(dst_filename)))
+	{
 		return -1;
+	}
 
 	// load src file
 	model = tifiles_file_get_model(src_filename);
 	type = tifiles_file_get_class(src_filename);
-	
+
 	te = tifiles_te_create(src_filename, type, model);
-	if(type == TIFILE_FLASH)
+	if (te == NULL)
+	{
+		ret = ERR_BAD_FILE;
+		goto ttaf;
+	}
+	if (type == TIFILE_FLASH)
 	{ 
 		ret = tifiles_file_read_flash(src_filename, te->content.flash);
-		if(ret) goto ttaf;
+		if (ret)
+		{
+			goto ttaf;
+		}
 	}
-	else if(type & TIFILE_REGULAR)
+	else if (type & TIFILE_REGULAR)
 	{ 
 		ret = tifiles_file_read_regular(src_filename, te->content.regular);
-		if(ret) goto ttaf;
+		if (ret)
+		{
+			goto ttaf;
+		}
 	}
 
 	// load dst file
 	content = tifiles_content_create_tigroup(CALC_NONE, 0);
 	ret = tifiles_file_read_tigroup(dst_filename, content);
-	if(ret) goto ttaf;
+	if (ret)
+	{
+		goto ttaf;
+	}
 
 	tifiles_content_add_te(content, te);
 	ret = tifiles_file_write_tigroup(dst_filename, content);
-	if(ret) goto ttaf;
+	if (ret)
+	{
+		goto ttaf;
+	}
 
 	tifiles_content_delete_tigroup(content);
 
 	return 0;
 
 ttaf:	// release on exit
-    tifiles_te_delete(te);
+	tifiles_te_delete(te);
 	tifiles_content_delete_tigroup(content);
 	return ret;
 }
@@ -409,7 +511,7 @@ ttaf:	// release on exit
 TIEXPORT2 int TICALL tifiles_tigroup_del_file(TigEntry *entry, const char *filename)
 {
 	TigContent* content = NULL;
-	int err = 0;
+	int ret = 0;
 
 	if (entry == NULL || filename == NULL)
 	{
@@ -417,17 +519,21 @@ TIEXPORT2 int TICALL tifiles_tigroup_del_file(TigEntry *entry, const char *filen
 		return -1;
 	}
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_tigroup_del_file: %p", entry);
+	tifiles_te_display(entry);
+#endif
+
 	content = tifiles_content_create_tigroup(CALC_NONE, 0);
-	err = tifiles_file_read_tigroup(filename, content);
-	if(err) goto tfdf;
+	ret = tifiles_file_read_tigroup(filename, content);
+	if (!ret)
+	{
+		(void)tifiles_content_del_te(content, entry);
+		ret = tifiles_file_write_tigroup(filename, content);
+	}
 
-	tifiles_content_del_te(content, entry);
-	err = tifiles_file_write_tigroup(filename, content);
-	if(err) goto tfdf;
-
-tfdf:
 	tifiles_content_delete_tigroup(content);
-	return 0;
+	return ret;
 }
 
 // ---------------------------------------------------------------------------
@@ -449,8 +555,10 @@ TIEXPORT2 int TICALL tifiles_tigroup_contents(FileContent **src_contents1, Flash
 	int i, m=0, n=0;
 	CalcModel model = CALC_NONE;
 
-	if(src_contents1 == NULL && src_contents2 == NULL)
+	if (src_contents1 == NULL && src_contents2 == NULL)
+	{
 		return -1;
+	}
 
 	if (dst_content == NULL)
 	{
@@ -458,23 +566,35 @@ TIEXPORT2 int TICALL tifiles_tigroup_contents(FileContent **src_contents1, Flash
 		return -1;
 	}
 
-	if(src_contents1)
+	if (src_contents1)
+	{
 		for (m = 0; src_contents1[m] != NULL; m++);
-	if(src_contents2)
+	}
+	if (src_contents2)
+	{
 		for (n = 0; src_contents2[n] != NULL; n++);
+	}
 
-	if(src_contents2)
-		if(*src_contents2)
+	if (src_contents2)
+	{
+		if (*src_contents2)
+		{
 			model = src_contents2[0]->model;
-	if(src_contents1)
-		if(*src_contents1)
+		}
+	}
+	if (src_contents1)
+	{
+		if (*src_contents1)
+		{
 			model = src_contents1[0]->model;	// FileContent is more precise than FlashContent
+		}
+	}
 
 	content = tifiles_content_create_tigroup(model, m+n);
 
-	if(src_contents1)
+	if (src_contents1)
 	{
-		for(i = 0; i < m; i++)
+		for (i = 0; i < m; i++)
 		{
 			TigEntry *te = (TigEntry *)g_malloc0(sizeof(TigEntry));
 
@@ -485,20 +605,31 @@ TIEXPORT2 int TICALL tifiles_tigroup_contents(FileContent **src_contents1, Flash
 		}
 	}
 
-	if(src_contents2)
+	if (src_contents2)
 	{
-		for(i = 0; i < n; i++)
+		for (i = 0; i < n; i++)
 		{
-			TigEntry *te = (TigEntry *)g_malloc0(sizeof(TigEntry));
+			TigEntry *te;
 			VarEntry ve;
 			FlashContent *ptr;
 
 			for (ptr = src_contents2[i]; ptr; ptr = ptr->next)
-				if(ptr->data_type == tifiles_flash_type(model))
+			{
+				if (ptr->data_type == tifiles_flash_type(model))
+				{
 					break;
-			
-			strcpy(ve.folder, "");
-			strcpy(ve.name, ptr->name);
+				}
+			}
+			if (ptr == NULL)
+			{
+				tifiles_critical("%s: ptr is NULL, skipping", __FUNCTION__);
+				continue;
+			}
+
+			te = (TigEntry *)g_malloc0(sizeof(TigEntry));
+			ve.folder[0] = 0;
+			strncpy(ve.name, ptr->name, sizeof(ve.name) - 1);
+			ve.name[sizeof(ve.name) - 1] = 0;
 			ve.type = ptr->data_type;
 			te->filename = tifiles_build_filename(model, &ve);
 			te->type = TIFILE_FLASH;
@@ -509,6 +640,11 @@ TIEXPORT2 int TICALL tifiles_tigroup_contents(FileContent **src_contents1, Flash
 
 	*dst_content = content;
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_tigroup_contents: %p", content);
+	tifiles_file_display_tigcontent(content);
+#endif
+
 	return 0;
 }
 
@@ -529,7 +665,7 @@ TIEXPORT2 int TICALL tifiles_untigroup_content(TigContent *src_content, FileCont
 	TigContent *src = src_content;
 	FileContent **dst1 = NULL;
 	FlashContent **dst2 = NULL;
-	int i, j;
+	unsigned int i, j;
 
 	if (src_content == NULL || dst_contents1 == NULL || dst_contents2 == NULL)
 	{
@@ -537,27 +673,33 @@ TIEXPORT2 int TICALL tifiles_untigroup_content(TigContent *src_content, FileCont
 		return -1;
 	}
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_untigroup_content: %p", src_content);
+	tifiles_file_display_tigcontent(src_content);
+#endif
+
 	// allocate an array of FileContent/FlashContent structures (NULL terminated)
+	dst1 = (FileContent **)g_malloc0((src->n_vars+1) * sizeof(FileContent *));
+	if (dst1 == NULL)
 	{
-		dst1 = (FileContent **)g_malloc0((src->n_vars+1) * sizeof(FileContent *));
-		if (dst1 == NULL)
-			return ERR_MALLOC;
+		return ERR_MALLOC;
 	}
+	dst2 = (FlashContent **)g_malloc0((src->n_apps+1) * sizeof(FlashContent *));
+	if (dst2 == NULL)
 	{
-		dst2 = (FlashContent **)g_malloc0((src->n_apps+1) * sizeof(FlashContent *));
-		if (dst2 == NULL)
-			return ERR_MALLOC;
+		g_free(dst1);
+		return ERR_MALLOC;
 	}
 
 	// parse each entry and duplicate it into a single content
-	for(i = 0; i < src->n_vars; i++)
+	for (i = 0; i < src->n_vars; i++)
 	{
 		TigEntry *te = src->var_entries[i];
 
 		dst1[i] = tifiles_content_dup_regular(te->content.regular);
 	}
 
-	for(j = 0; j < src->n_apps; j++)
+	for (j = 0; j < src->n_apps; j++)
 	{
 		TigEntry *te = src->app_entries[j];
 
@@ -595,54 +737,75 @@ TIEXPORT2 int TICALL tifiles_tigroup_files(char **src_filenames, const char *dst
 	}
 
 	// counts number of files to group and allocate space for that
-	for(k = m = n = 0; src_filenames[k]; k++)
+	for (k = m = n = 0; src_filenames[k]; k++)
 	{
-		if(tifiles_file_is_regular(src_filenames[k]))
+		if (tifiles_file_is_regular(src_filenames[k]))
+		{
 			m++;
-		else if(tifiles_file_is_flash(src_filenames[k]))
+		}
+		else if (tifiles_file_is_flash(src_filenames[k]))
+		{
 			n++;
+		}
 	}
 	model = tifiles_file_get_model(src_filenames[0]);
 
 	// allocate space for that
 	src1 = (FileContent **)g_malloc0((m + 1) * sizeof(FileContent *));
 	if (src1 == NULL)
+	{
 		return ERR_MALLOC;
+	}
 
 	src2 = (FlashContent **)g_malloc0((n + 1) * sizeof(FlashContent *));
 	if (src2 == NULL)
-		return ERR_MALLOC;
-
-	for(i = j = k = 0; k < m+n; k++)
 	{
-		if(tifiles_file_is_regular(src_filenames[k]))
+		g_free(src1);
+		return ERR_MALLOC;
+	}
+
+	for (i = j = k = 0; k < m+n; k++)
+	{
+		if (tifiles_file_is_regular(src_filenames[k]))
 		{
 			src1[i] = tifiles_content_create_regular(model);
 			ret = tifiles_file_read_regular(src_filenames[k], src1[i]);
-			if(ret) goto tgf;
+			if (ret)
+			{
+				goto tgf;
+			}
 			i++;
 		}
-		else if(tifiles_file_is_flash(src_filenames[k]))
+		else if (tifiles_file_is_flash(src_filenames[k]))
 		{
 			src2[j] = tifiles_content_create_flash(model);
 			ret = tifiles_file_read_flash(src_filenames[k], src2[j]);
-			if(ret) goto tgf;
+			if (ret)
+			{
+				goto tgf;
+			}
 			j++;
 		}
 	}
 
 	ret = tifiles_tigroup_contents(src1, src2, &dst);
-	if(ret) goto tgf;
+	if (ret)
+	{
+		goto tgf;
+	}
 
 	ret = tifiles_file_write_tigroup(dst_filename, dst);
-	if(ret) goto tgf;
 
 tgf:
-	for(i = 0; i < m; i++)
+	for (i = 0; i < m; i++)
+	{
 		g_free(src1[i]);
+	}
 	g_free(src1);
-	for(i = 0; i < n; i++)
+	for (i = 0; i < n; i++)
+	{
 		g_free(src2[i]);
+	}
 	g_free(src2);
 	tifiles_content_delete_tigroup(dst);
 
@@ -669,7 +832,7 @@ TIEXPORT2 int TICALL tifiles_untigroup_file(const char *src_filename, char ***ds
 	FileContent **ptr1, **dst1 = NULL;
 	FlashContent **ptr2, **dst2 = NULL;
 	char *real_name;
-	int i, j;
+	unsigned int i, j;
 	int ret = 0;
 
 	if (src_filename == NULL)
@@ -681,50 +844,76 @@ TIEXPORT2 int TICALL tifiles_untigroup_file(const char *src_filename, char ***ds
 	// read TiGroup file
 	src = tifiles_content_create_tigroup(CALC_NONE, 0);
 	ret = tifiles_file_read_tigroup(src_filename, src);
-	if(ret) goto tuf;
+	if (ret)
+	{
+		goto tuf;
+	}
 
 	// ungroup structure
 	ret = tifiles_untigroup_content(src, &dst1, &dst2);
-	if(ret) goto tuf;
+	if (ret)
+	{
+		goto tuf;
+	}
 
 	// count number of structures and allocates array of strings
-	if(dst_filenames != NULL)
+	if (dst_filenames != NULL)
+	{
 		*dst_filenames = (char **)g_malloc((src->n_vars + src->n_apps + 1) * sizeof(char *));
+	}
 
 	// store each structure content to file
 	for (ptr1 = dst1, i = 0; *ptr1 != NULL || i < src->n_vars; ptr1++, i++)
 	{
 		ret = tifiles_file_write_regular(NULL, *ptr1, &real_name);
-		if(ret) goto tuf;
+		if (ret)
+		{
+			goto tuf;
+		}
 
-		if(dst_filenames != NULL)
+		if (dst_filenames != NULL)
+		{
 			*dst_filenames[i] = real_name;
+		}
 		else
+		{
 			g_free(real_name);
+		}
 	}
 
 	for (ptr2 = dst2, j = 0; *ptr2 != NULL || j < src->n_apps; ptr2++, j++)
 	{
 		ret = tifiles_file_write_flash2(NULL, *ptr2, &real_name);
-		if(ret) goto tuf;
+		if (ret)
+		{
+			goto tuf;
+		}
 
-		if(dst_filenames != NULL)
+		if (dst_filenames != NULL)
+		{
 			*dst_filenames[i+j] = real_name;
+		}
 		else
+		{
 			g_free(real_name);
+		}
 	}
 
 	// release allocated memory
 tuf:
-	if(dst1)
+	if (dst1)
 	{
-		for(ptr1 = dst1; *ptr1; ptr1++)
+		for (ptr1 = dst1; *ptr1; ptr1++)
+		{
 			tifiles_content_delete_regular(*ptr1);
+		}
 	}
-	if(dst2)
+	if (dst2)
 	{
-		for(ptr2 = dst2; *ptr2; ptr2++)
+		for (ptr2 = dst2; *ptr2; ptr2++)
+		{
 			tifiles_content_delete_flash(*ptr2);
+		}
 	}
 	tifiles_content_delete_tigroup(src);
 
@@ -743,9 +932,9 @@ tuf:
  *
  * Return value: the allocated block.
  **/
-TIEXPORT2 TigContent* TICALL tifiles_content_create_tigroup(CalcModel model, int n)
+TIEXPORT2 TigContent* TICALL tifiles_content_create_tigroup(CalcModel model, unsigned int n)
 {
-	TigContent* content = g_malloc0(sizeof(FileContent));
+	TigContent* content = g_malloc0(sizeof(*content));
 	if (content != NULL)
 	{
 		content->model = content->model_dst = model;
@@ -755,6 +944,11 @@ TIEXPORT2 TigContent* TICALL tifiles_content_create_tigroup(CalcModel model, int
 		content->app_entries = (TigEntry **)g_malloc0(sizeof(TigEntry *));
 	}
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_content_create_tigroup: %p", content);
+	tifiles_file_display_tigcontent(content);
+#endif
+
 	return content;
 }
 
@@ -767,21 +961,37 @@ TIEXPORT2 TigContent* TICALL tifiles_content_create_tigroup(CalcModel model, int
  **/
 TIEXPORT2 int TICALL tifiles_content_delete_tigroup(TigContent *content)
 {
-	int i;
+	unsigned int i;
+
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_content_delete_tigroup: %p", content);
+	tifiles_file_display_tigcontent(content);
+#endif
 
 	if (content != NULL)
 	{
-		for(i = 0; i < content->n_vars; i++)
+		if (content->var_entries != NULL)
 		{
-			TigEntry* entry = content->var_entries[i];
-			tifiles_te_delete(entry);
+			for (i = 0; i < content->n_vars; i++)
+			{
+				TigEntry* entry = content->var_entries[i];
+				tifiles_te_delete(entry);
+			}
+			g_free(content->var_entries);
 		}
 
-		for(i = 0; i < content->n_apps; i++)
+		if (content->app_entries != NULL)
 		{
-			TigEntry* entry = content->app_entries[i];
-			tifiles_te_delete(entry);
+			for (i = 0; i < content->n_apps; i++)
+			{
+				TigEntry* entry = content->app_entries[i];
+				tifiles_te_delete(entry);
+			}
+			g_free(content->app_entries);
 		}
+
+		g_free(content->comment);
+
 		g_free(content);
 	}
 	else
@@ -802,7 +1012,9 @@ static int open_temp_file(const char *orig_name, char **temp_name)
 	*temp_name = NULL;
 	suffix = strrchr(orig_name, '.');
 	if (suffix && (strchr(suffix, '/') || strchr(suffix, '\\')))
+	{
 		suffix = NULL;
+	}
 	template = g_strconcat("tigXXXXXX", suffix, NULL);
 
 	fd = g_file_open_tmp(template, temp_name, NULL);
@@ -822,6 +1034,7 @@ static int open_temp_file(const char *orig_name, char **temp_name)
  * @content: where to store content (may be re-allocated).
  *
  * This function loads a TiGroup from \a filename and places its content into \a content.
+ * If an error occurs, the structure content is *NOT* released for you.
  *
  * The temporary folder is used by this function to store temporary files.
  * 
@@ -833,7 +1046,7 @@ TIEXPORT2 int TICALL tifiles_file_read_tigroup(const char *filename, TigContent
 	struct archive *arc;
 	struct archive_entry *entry;
 	const char *filename_inzip;
-	int err = 0;
+	int ret = 0;
 
 	if (filename == NULL || content == NULL)
 	{
@@ -853,7 +1066,9 @@ TIEXPORT2 int TICALL tifiles_file_read_tigroup(const char *filename, TigContent
 	    || archive_read_open_FILE(arc, tigf) != ARCHIVE_OK)
 	{
 		if (arc)
-			archive_read_finish(arc);
+		{
+			archive_read_free(arc);
+		}
 		fclose(tigf);
 		return ERR_FILE_ZIP;
 	}
@@ -889,7 +1104,7 @@ TIEXPORT2 int TICALL tifiles_file_read_tigroup(const char *filename, TigContent
 		fd = open_temp_file(filename_inzip, &fname);
 		if (fd == -1)
 		{
-			err = ERR_FILE_IO;
+			ret = ERR_FILE_IO;
 			goto tfrt_exit;
 		}
 
@@ -899,7 +1114,7 @@ TIEXPORT2 int TICALL tifiles_file_read_tigroup(const char *filename, TigContent
 			close(fd);
 			g_unlink(fname);
 			g_free(fname);
-			err = ERR_FILE_IO;
+			ret = ERR_FILE_IO;
 			goto tfrt_exit;
 		}
 		close(fd);
@@ -908,28 +1123,46 @@ TIEXPORT2 int TICALL tifiles_file_read_tigroup(const char *filename, TigContent
 		{
 			int model = tifiles_file_get_model(fname);
 
-			if(content->model == CALC_NONE)
+			if (content->model == CALC_NONE)
+			{
 				content->model = model;
-
-			if(tifiles_file_is_regular(fname))
-			{
-				TigEntry *tigentry = tifiles_te_create(filename_inzip, tifiles_file_get_class(fname), content->model);
-				int ret;
-
-				ret = tifiles_file_read_regular(fname, tigentry->content.regular);
-				if(ret) { g_free(tigentry); g_unlink(fname); g_free(fname); err = ret; goto tfrt_exit; }
-
-				tifiles_content_add_te(content, tigentry);
 			}
-			else if(tifiles_file_is_flash(fname))
+
+			if (tifiles_file_is_regular(fname))
 			{
 				TigEntry *tigentry = tifiles_te_create(filename_inzip, tifiles_file_get_class(fname), content->model);
-				int ret;
 
-				ret = tifiles_file_read_flash(fname, tigentry->content.flash);
-				if(ret) { g_free(tigentry); g_unlink(fname); g_free(fname); err = ret; goto tfrt_exit; }
+				if (tigentry != NULL)
+				{
+					ret = tifiles_file_read_regular(fname, tigentry->content.regular);
+					if (ret)
+					{
+						g_free(tigentry);
+						g_unlink(fname);
+						g_free(fname);
+						break;
+					}
 
-				tifiles_content_add_te(content, tigentry);
+					tifiles_content_add_te(content, tigentry);
+				}
+			}
+			else if (tifiles_file_is_flash(fname))
+			{
+				TigEntry *tigentry = tifiles_te_create(filename_inzip, tifiles_file_get_class(fname), content->model);
+
+				if (tigentry != NULL)
+				{
+					ret = tifiles_file_read_flash(fname, tigentry->content.flash);
+					if (ret)
+					{
+						g_free(tigentry);
+						g_unlink(fname);
+						g_free(fname);
+						break;
+					}
+
+					tifiles_content_add_te(content, tigentry);
+				}
 			}
 			else
 			{
@@ -940,11 +1173,16 @@ TIEXPORT2 int TICALL tifiles_file_read_tigroup(const char *filename, TigContent
 		g_free(fname);
 	}
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_file_read_tigroup: %p", content);
+	tifiles_file_display_tigcontent(content);
+#endif
+
 	// Close
 tfrt_exit:
-	archive_read_finish(arc);
+	archive_read_free(arc);
 	fclose(tigf);
-	return err;
+	return ret;
 }
 
 static int zip_write(struct archive *arc, CalcModel model, const char *origfname, const char *tempfname)
@@ -990,13 +1228,15 @@ static int zip_write(struct archive *arc, CalcModel model, const char *origfname
 	{
 		tifiles_critical("zip_write: cannot read temporary file");
 		archive_entry_free(entry);
-		return ERR_FILE_IO;
+		err = ERR_FILE_IO;
+		goto end2;
 	}
 
 	if (archive_write_header(arc, entry) != ARCHIVE_OK)
 	{
 		archive_entry_free(entry);
-		return ERR_FILE_IO;
+		err = ERR_FILE_IO;
+		goto end;
 	}
 	archive_entry_free(entry);
 
@@ -1027,7 +1267,9 @@ static int zip_write(struct archive *arc, CalcModel model, const char *origfname
 	} while (!err && (size_read>0));
 
 	g_free(buf);
+end:
 	fclose(f);
+end2:
 	return err;
 }
 
@@ -1056,6 +1298,11 @@ TIEXPORT2 int TICALL tifiles_file_write_tigroup(const char *filename, TigContent
 		return -1;
 	}
 
+#ifdef TRACE_CONTENT_INSTANCES
+	tifiles_info("tifiles_file_write_tigroup: %p", content);
+	tifiles_file_display_tigcontent(content);
+#endif
+
 	// Open ZIP archive
 	tigf = g_fopen(filename, "wb");
 	if (tigf == NULL)
@@ -1065,9 +1312,10 @@ TIEXPORT2 int TICALL tifiles_file_write_tigroup(const char *filename, TigContent
 
 	if (!(arc = archive_write_new()) || archive_write_set_format_zip(arc) != ARCHIVE_OK)
 	{
-		if (arc) {
+		if (arc)
+		{
 			archive_write_close(arc);
-			archive_write_finish(arc);
+			archive_write_free(arc);
 		}
 		fclose(tigf);
 		return ERR_FILE_OPEN;
@@ -1078,15 +1326,21 @@ TIEXPORT2 int TICALL tifiles_file_write_tigroup(const char *filename, TigContent
 	archive_write_set_bytes_per_block(arc, 0);
 
 	if (content->comp_level > 0)
+	{
 		archive_write_set_options(arc, "compression=deflate");
+	}
 	else
+	{
 		archive_write_set_options(arc, "compression=store");
+	}
 
 	if (archive_write_open_FILE(arc, tigf) != ARCHIVE_OK)
+	{
 		err = ERR_FILE_OPEN;
+	}
 
 	// Parse entries and store
-	for(ptr = content->var_entries; *ptr && !err; ptr++)
+	for (ptr = content->var_entries; *ptr && !err; ptr++)
 	{
 		TigEntry* entry = *ptr;
 		char *fname = NULL;
@@ -1094,7 +1348,8 @@ TIEXPORT2 int TICALL tifiles_file_write_tigroup(const char *filename, TigContent
 
 		// write TI file into tmp folder
 		fd = open_temp_file(entry->filename, &fname);
-		if (fd == -1) {
+		if (fd == -1)
+		{
 			g_free(fname);
 			err = ERR_FILE_OPEN;
 			break;
@@ -1103,13 +1358,15 @@ TIEXPORT2 int TICALL tifiles_file_write_tigroup(const char *filename, TigContent
 
 		err = tifiles_file_write_regular(fname, entry->content.regular, NULL);
 		if (!err)
+		{
 			err = zip_write(arc, content->model, entry->filename, fname);
+		}
 
 		g_unlink(fname);
 		g_free(fname);
 	}
 
-	for(ptr = content->app_entries; *ptr && !err; ptr++)
+	for (ptr = content->app_entries; *ptr && !err; ptr++)
 	{
 		TigEntry* entry = *ptr;
 		char *fname = NULL;
@@ -1117,7 +1374,8 @@ TIEXPORT2 int TICALL tifiles_file_write_tigroup(const char *filename, TigContent
 
 		// write TI file into tmp folder
 		fd = open_temp_file(entry->filename, &fname);
-		if (fd == -1) {
+		if (fd == -1)
+		{
 			g_free(fname);
 			err = ERR_FILE_OPEN;
 			break;
@@ -1126,7 +1384,9 @@ TIEXPORT2 int TICALL tifiles_file_write_tigroup(const char *filename, TigContent
 
 		err = tifiles_file_write_flash(fname, entry->content.flash);
 		if (!err)
+		{
 			err = zip_write(arc, content->model, entry->filename, fname);
+		}
 
 		g_unlink(fname);
 		g_free(fname);
@@ -1134,8 +1394,10 @@ TIEXPORT2 int TICALL tifiles_file_write_tigroup(const char *filename, TigContent
 
 	// close archive
 	if (archive_write_close(arc) != ARCHIVE_OK)
+	{
 		err = ERR_FILE_IO;
-	archive_write_finish(arc);
+	}
+	archive_write_free(arc);
 	fclose(tigf);
 	return err;
 }
@@ -1162,14 +1424,18 @@ TIEXPORT2 int TICALL tifiles_file_display_tigroup(const char *filename)
 
 	tigf = g_fopen(filename, "rb");
 	if (tigf == NULL)
+	{
 		return ERR_FILE_OPEN;
+	}
 
 	if (!(arc = archive_read_new())
 	    || archive_read_support_format_zip(arc) != ARCHIVE_OK
 	    || archive_read_open_FILE(arc, tigf) != ARCHIVE_OK)
 	{
 		if (arc)
-			archive_read_finish(arc);
+		{
+			archive_read_free(arc);
+		}
 		fclose(tigf);
 		return ERR_FILE_ZIP;
 	}
@@ -1188,7 +1454,56 @@ TIEXPORT2 int TICALL tifiles_file_display_tigroup(const char *filename)
 		g_free(dispname);
 	}
 
-	archive_read_finish(arc);
+	archive_read_free(arc);
 	fclose(tigf);
 	return 0;
 }
+
+/**
+ * tifiles_file_display_tigcontent:
+ * @content: the tigroup content to show, TigContent pointer.
+ *
+ * Display tigroup information contained in a TigContent structure.
+ *
+ * Return value: an error code, 0 otherwise.
+ **/
+TIEXPORT2 int TICALL tifiles_file_display_tigcontent(TigContent *content)
+{
+	unsigned int i;
+
+	if (content == NULL)
+	{
+		tifiles_critical("%s(NULL)", __FUNCTION__);
+		return ERR_INVALID_FILE;
+	}
+
+	tifiles_info("Model:             %02X (%u)", content->model, content->model);
+	tifiles_info("Signature:         %s", tifiles_calctype2signature(content->model));
+	tifiles_info("model_dst:         %02X (%u)", content->model_dst, content->model_dst);
+	tifiles_info("Comment:           %s", content->comment);
+	tifiles_info("Compression level: %d", content->comp_level);
+
+	tifiles_info("Number of vars:    %u", content->n_vars);
+	tifiles_info("Var entries:       %p", content->var_entries);
+
+	if (content->var_entries != NULL)
+	{
+		for (i = 0; i < content->n_vars; i++)
+		{
+			tifiles_te_display(content->var_entries[i]);
+		}
+	}
+
+	tifiles_info("Number of apps:    %u", content->n_apps);
+	tifiles_info("Apps entries:      %p", content->app_entries);
+
+	if (content->app_entries != NULL)
+	{
+		for (i = 0; i < content->n_apps; i++)
+		{
+			tifiles_te_display(content->app_entries[i]);
+		}
+	}
+
+	return 0;
+}
