@@ -45,16 +45,4 @@
 #define TI92_VNONE 0
 #define TI92_VLOCK 1
 
-// libtifiles: for internal use only, not exported !
-
-extern const char *TI92_CONST[TI92_MAXTYPES + 1][4];
-
-const char *ti92_byte2type(uint8_t data);
-uint8_t ti92_type2byte(const char *s);
-const char *ti92_byte2fext(uint8_t data);
-uint8_t ti92_fext2byte(const char *s);
-
-const char *ti92_byte2desc(uint8_t data);
-const char *ti92_byte2icon(uint8_t data);
-
 #endif
